% SRB_RUN_SR_SELECT

%OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
%OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO----Parameters----OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO
%OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO

clear parameters;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%+++++++++++IMG_1506
% folder containing the tracked sequence
inputFolder = 'data\IMG_0663_all_tracked\';
parameters.epsilon_recon = 0.001;%0.01;  % the smaller the epsilon, the more cartoon-like result
%........................................
parameters.temporalRadius = 15;%9;%15;   % use 2*temporalRadius+1 frames to super-resolve one frame
parameters.blurKerRadius = 15;%25;%15;
parameters.blurKerSmoothness = 5;
parameters.dataTermScale_recon = 6;%8;%15;%5;%3;
parameters.killSmoothFrames = 20; %25; %12;
parameters.numOuterIters = 3;
parameters.numIRLSIters_recon = 2;
parameters.initGaussSigma = 0.8;
parameters.celiuOFlowAlpha = 0.02;
parameters.scaleFactor = 2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%+++++++++++++++++++

parameters.bEstimateOpticalFlow = true;

parameters.bEstimateKernel = true;

parameters.eta = 0.02;

%OOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO

%---------------------------------------------------------------------------------------------------------

if inputFolder(end) ~= '\'
    inputFolder = [inputFolder,'\'];
end

parameters.totalFrames = length(srb_img_dir(inputFolder));

if parameters.bEstimateKernel
    resultFolder = sprintf('%sresults\\withKerEst', inputFolder);
    
    strInfo = sprintf(...
        'result[tr=%03d,kr=%03d,ks=%04.1f,dts=%03d,kill=%02d,iter[%02d,%02d],sig0=%2.1f,a=%5.3f]', ...
        parameters.temporalRadius, ...      % tr
        parameters.blurKerRadius, ...       % kr
        parameters.blurKerSmoothness, ...   % ks
        parameters.dataTermScale_recon, ... % dts
        parameters.killSmoothFrames, ...    % kill
        parameters.numOuterIters, ...       % iter[outerIters, ...]
        parameters.numIRLSIters_recon, ...  % iter[..., IRLSIters_recon]
        parameters.initGaussSigma, ...      % sig0
        parameters.celiuOFlowAlpha);        % a
else
    resultFolder = sprintf('%sresults\\withoutKerEst', inputFolder);
    
    strInfo = sprintf('result[tr=%03d,dts=%03d,kill=%02d,iter[%02d,%02d],sig0=%2.1f,a=%5.3f]', ...
        parameters.temporalRadius, ...      % tr
        parameters.dataTermScale_recon, ... % dts
        parameters.killSmoothFrames,...     % kill
        parameters.numOuterIters, ...       % iter[outerIters, ...]
        parameters.numIRLSIters_recon, ...  % iter[..., IRLSIters_recon]
        parameters.initGaussSigma, ...      % sig0
        parameters.celiuOFlowAlpha);        % a
end

if ~exist(resultFolder,'dir')
    mkdir(resultFolder);
end

resultFolder = sprintf('%s\\%s\\', resultFolder, strInfo);

if ~exist(resultFolder,'dir')
    mkdir(resultFolder);
end

parameters.inputFolder  = inputFolder ;
parameters.resultFolder = resultFolder;

%---------------------------------------------------------------------------------------------------------

global iItersInIRLS;
global imgGroundTruth;

iItersInIRLS = 0;

if ~isfield(parameters,'imgGroundTruth')
    imgGroundTruth = [];
else
    imgGroundTruth = parameters.imgGroundTruth;
end

%---------------------------------------------------------------------------------------------------------

stream = RandStream.getGlobalStream; reset(stream);  %% for DEBUGGING, because MSAC uses random selection

temporalRadius = parameters.temporalRadius;

%for ii = 1 : parameters.totalFrames
ii = parameters.temporalRadius + 1;
    
    % load frames: vecImgJ for the ii-th frame in the input folder
    [vecImgJ,idx0] = srb_loadframes_for_frame(ii, inputFolder, temporalRadius, false, false); % No hom, No oflow!!!
    
    imwrite(vecImgJ{idx0}, [resultFolder, 'input.png']);
%     imgLanczos4 = srb_imresize(vecImgJ{idx0},parameters.scaleFactor*size(vecImgJ{idx0}),'lanczos4');
%     imwrite(imgLanczos4, [resultFolder, 'input_lanczos4.png']);
    
    imgNN = srb_imresize(vecImgJ{idx0},parameters.scaleFactor*size(vecImgJ{idx0}),'nearest');
    imwrite(imgNN, [resultFolder, 'input_NN.png']);
    
    imgBicubic = srb_imresize(vecImgJ{idx0},parameters.scaleFactor*size(vecImgJ{idx0}),'bicubic');
    imwrite(imgBicubic, [resultFolder, 'input_bicubic.png']);
    
    parameters.currentFrame = ii;
    
    [imgI,~,parameters] = srb_multiframeSR_select(vecImgJ, idx0, parameters);  %%%%%%%%%%%%%%%%%%%%%%%%% CORE
    
%     imgIMed = srb_median_filter(imgI, [7,7]);
%     imgIBilat = sm_bilateralfilter(imgI,imgI,10,10,0.05);

%end  %ii

%% save and show result

imwrite(imgI, [resultFolder, strInfo, '.png']);
% imwrite(imgIMed, [resultFolder, strInfo, '_med.png']);
% imwrite(imgIBilat, [resultFolder, strInfo, '_bilat.png']);

fprintf('Elapsed times: optical flow = %7.2f\n', parameters.timeOpticalFlow);
fprintf('Elapsed times: image reconstruction = %7.2f\n', parameters.timeImageReconstruction);
fprintf('Elapsed times: kernel estimation = %7.2f\n', parameters.timeKernelEstimation);

%figure;imshow(imgLanczos4); title('Lanczos4 of input:');
figure;imshow(imgI); title(['final\_', strInfo, ':']);
%---------------------------------------------------------------------------------------------------------
